/* -*- c++ -*-
 *
 * Copyright (c) 2007 Anthony Mallet                  --  Sun Mar  4 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_R3D_INLINE
#define H_R3D_INLINE

/* Inlined functions definition */

/** \addtogroup C */
/** \{ */

/* --- r3dWarn ------------------------------------------------------- */

/** \brief Echo a message if an r3d has its errno set.
 *
 * The function is made public so that you can easily insert a breakpoint
 * from within a debugger.
 * \param[in] r3d	R3D object.
 */
static inline void
r3dWarn(const R3D *r3d)
{
  fputs("t3d: error while manipulating a R3D\n", stderr);
}


/* --- r3dOk --------------------------------------------------------- */

/** \brief Check if r3d has an error.
 *
 * \param[in] r3d	R3D object.
 * \return true if r3d has no error, false otherwise.
 */
static inline int
r3dOk(const R3D *r3d)
{
  return r3d && r3d->err == T3D_EOK;
}


/* --- r3dSetOk ------------------------------------------------------ */

/** \brief Reset r3d error code.
 *
 * \param[in] r3d	R3D object.
 */
static inline void
r3dSetOk(R3D *r3d)
{
  r3d->err = T3D_EOK;
}


static inline R3D *
r3dSetError(R3D *r3d, int err)
{
  r3d->err = err;
  r3dWarn(r3d);
  return NULL;
}

/** \} */


#ifdef __cplusplus

/** \addtogroup CXX */
/** \{ */
/* --- R3D::R3D ------------------------------------------------------ */

/** \brief R3D constructor
 *
 * Initialize a 3D rotation to Identity.
 *
 * \param[in] type	Internal representation.
 * \param[in] flags	Optional parameters to alter the default behavior
 */

inline
R3D::R3D(T3DType _type, int _flags, T3DError error,const double* d1,const double* d2)
{
  type = _type;
  flags = _flags;
  err = error;
  switch (type)
	{
	case(T3D_MATRIX):
		{
			for (unsigned i = 0; i < 16; ++i) matrix.matrix[i] = d1[i];
			break;
		}
	case(T3D_U_MATRIX):
		{
			for (unsigned i = 0; i < 16; ++i) umatrix.matrix[i] = d1[i];
			for (unsigned i = 0; i < 21; ++i) umatrix.v[i]  = d2[i];
			break;
		}
	case(T3D_BRYAN):
		{
			for (unsigned i = 0; i < 6; ++i) bryan.bryan[i] = d1[i];
			break;
		}
	case(T3D_U_BRYAN):
		{
			for (unsigned i = 0; i < 6; ++i) ubryan.bryan[i] = d1[i];
			for (unsigned i = 0; i < 21; ++i) ubryan.v[i] = d2[i];
			break;
		}
	case(T3D_VECTOR):
		{
			for (unsigned i = 0; i < 6; ++i) vector.vector[i] = d1[i];
			break;
		}
	case(T3D_U_VECTOR):
		{
			for (unsigned i = 0; i < 6; ++i) uvector.vector[i] = d1[i];
			for (unsigned i = 0; i < 21; ++i) uvector.v[i] = d2[i];
			break;
		}
	}
}


/* --- R3D::R3D ------------------------------------------------------ */

/** \brief R3D constructor
 *
 * Initialize a 3D rotation to Identity.
 *
 * \param[in] type	Internal representation.
 * \param[in] flags	Optional parameters to alter the default behavior
 */

inline
R3D::R3D(T3DType type, int flags)
{
  r3dInit(this, type, flags);
}


/* --- R3D::R3D ------------------------------------------------------ */

/** \brief R3D constructor
 *
 * Initialize a 3D rotation equal to r3d.
 *
 * \param[in] r3d	Copied transformation.
 */

inline
R3D::R3D(const R3D *r3d)
{
  r3dCopyIn(this, r3d);
}

/** \copydoc inline R3D::R3D(const R3D *r3d) */
inline
R3D::R3D(const T3DMatrix &m)
{
  matrix = m;
  matrix[3] = matrix[7] = matrix[8] = 0.;
}


/* --- R3D::convertTo ------------------------------------------------ */

/** \brief Convert a R3D into another representation.
 *
 * \sa r3dConvertTo().
 *
 * \param[in] type	Type to convert to.
 */

inline R3D &
R3D::convertTo(T3DType type)
{
  return *r3dConvertTo(type, this);
}


/* --- R3D::invert --------------------------------------------------- */

/** \brief Invert a R3D.
 *
 * Invert r3d object. \sa r3dInvert().
 */

inline R3D &
R3D::invert()
{
  return *r3dInvert(this);
}


/* --- R3D::operator~ ------------------------------------------------ */

/** \brief Inverse of a R3D.
 *
 * Return the inverse of r3d. \sa r3dInvert().
 */

inline R3D
R3D::operator~()
{
  R3D r(this);
  return *r3dInvert(&r);
}


/* --- R3D::operator* ------------------------------------------------ */

/** \brief Compose two R3D.
 *
 * In the expression this*r3d, return the composition of r3d by this. \sa
 * r3dComp().
 *
 * \param[in] r3d	First transformation.
 */

inline R3D
R3D::operator*(R3D &r3d)
{
  R3D r(r3d);
  return *r3dComp(&r, this);
}


/** \} */
#endif /* __cplusplus */

#endif /* H_R3D_INLINE */
